
'*******************************************************************
'*  Copyright 1988 (c)   Faulkner Consulting - All Rights Reserved *
'*                                                                 *
'*  Mean 18 Golf course selector uses PKWARE to store courses,     *
'*  saved games and hall of fames. It was written in Turbo Basic   *
'*  version 1.0 from Borland International.                        *
'*******************************************************************
'*  This program is copyrighted material and may not be modified   *
'*  or sold by anyone at anytime without prior written permission  *
'*  from Faulkner Consulting.  However if you choose to use this   *
'*  program you do so at your own risk!  Source code is provided   *
'*  so programmers can see how the SHELL command can be used to    *
'*  temporarily transfer control to other programs.                *
'*                                                                 *
'*                Address all Correspondance to:                   *
'*                                                                 *
'*                    FAULKNER CONSULTING                          *
'*                    ATTN: M18 Development                        *
'*                    4552 N. 76th Street                          *
'*                    Milwaukee, Wisconsin  53218                  *
'*******************************************************************

'=================================================================
Constants:

defint a-z                          'All vars are integer unless redefined
dim CourseNames$(400)               'Maximum of 400 Course Names

Max        = 400                    'Max Courses Array Size
true       = -1                     'Boolean TRUE
false      = 0                      'Boolean FALSE
Version$   = "v1.20"                'M18 version number
null$      = chr$(0)                'Null Character
UpArrow$   = null$+"H"              'Up Arrow Key
DnArrow$   = null$+"P"              'Down Arrow Key
LftArrow$  = null$+"K"              'Left Arrow Key
RgtArrow$  = null$+"M"              'Right Arrow Key
EscKey$    = chr$(27)               'Escape Key
RetKey$    = chr$(13)               'Return (ENTER) Key
EndKey$    = null$+"O"              'END Key
PgUp$      = null$+"I"              'Page Up Key
PgDn$      = null$+"Q"              'Page Down Key
F1$        = null$+";"              'F1 Key
F10$       = null$+"D"              'F10 Key

'=================================================================
Defaults:

Border       = 1                    'Blue Border
Background   = 1                    'Blue Screen
Forground    = 7                    'White Letters
Highcolor    = 15                   'High Intensity White
Blnkcolor    = 23                   'Blink White Letters
ArcName$     = "COURSES.ARC"        'Golf Courses Archive File
DirName$     = "M18.DIR"            'File that holds directory
TempName$    = "M18.TMP"            'Temporary File
Page         = 0                    'Default page#
Position     = 1                    'Default course#
LastSelected = 1                    'Default course#
DoAllFiles   = true                 'Update all files into Archive

'==================================================================
MainRoutine:

on error goto IgnoreErrors                       'Setup Error Trap
if Command$<>"" then ArcName$=ucase$(Command$)   'Get User supplied ARC

locate ,,0                                       'Turn OFF Cursor

gosub SetupScreen                                'Display Headings
gosub CopyRightScreen                            'Display Copyright
gosub GetCourseNames                             'Read ARC Directory

choice$ = ""                                     'Main Routine Loop
while (choice$<>EscKey$)
    choice$ = ""
    gosub SetupScreen
    gosub DisplayPage

    while (choice$<>EscKey$) and (choice$<>EndKey$)
        Blink = true
        gosub BlinkPosition
        gosub GetChar

        select case choice$
            case UpArrow$                        'Move up one course
                 Blink=false
                 gosub BlinkPosition
                 if position > 1 then Position=Position-1_
                 else Position=80

            case DnArrow$                        'Move down one course
                 Blink=false
                 gosub BlinkPosition
                 if position < 80 then Position=Position+1_
                 else Position=1

            case LftArrow$                       'Move left one column
                 Blink=false
                 gosub BlinkPosition
                 if position > 20 then Position=Position-20_
                 else Position=Position+60

            case RgtArrow$                       'Move right one column
                 Blink=False
                 gosub BlinkPosition
                 if position < 61 then Position=Position+20_
                 else Position=Position-60

            case EscKey$                         'EXIT TO DOS

            case RetKey$                         'Select a course to play
                 Reverse=false
                 gosub ReverseCourse

                 LastSelected = position
                 Reverse=true
                 gosub ReverseCourse

            case EndKey$                         'SHELL to GOLF

            case PgUp$                           'Display next 80 courses
                 if page < 4 then page=page+1 else page=0
                 gosub DisplayPage

            case PgDn$                           'Display prevous 80 courses
                 if page > 0 then page=page-1 else page=4
                 gosub DisplayPage

            case F1$                             'Display Help Screen
                 gosub DisplayHelp
                 gosub SetupScreen
                 gosub DisplayPage

            case F10$                            'Update Courses array
                 DoAllFiles=true                 'Update all files
                 gosub UpdateArchive
                 gosub SetupScreen
                 gosub GetCourseNames
                 gosub DisplayPage
        end select
    wend

    if choice$ = EndKey$ then                    'Lets play Golf?
        i = (page*80)+LastSelected
        j = instr(CourseNames$(i),"(")

        if j = 0 then                            'No Course Selected
            Beep 2
            page = 0                             'point at 1st page
            LastSelected = 1                     'reset selected
            position = 1                         '1st course
        else
            color Highcolor,Background,Border    'Highlight Message
            call PrintString(25,30," LOADING COURSE ")

            ShellCommand$ = "PKXARC "+ArcName$+" "+_
                mid$(CourseNames$(i),5,j-6)+".* >"+TempName$
            shell ShellCommand$
            kill TempName$

            shell "GOLF"                         'Play Golf

            DoAllFiles=false                     'Don't save *.M18
            gosub UpdateArchive                  'Do Update
            gosub GetCourseNames                 'Update Array Values
        end if
    end if
wend
goto ExitRoutine

'==================================================================
SetupScreen:

color Forground,Background,Broder                'setup screen colors
cls                                              'clear screen

call PrintString(1,1,"ESC=Exit")

color Background,Forground,Border
call PrintString(1,24," M18 GOLF COURSE SELECTOR "+version$)

color Forground,Background,Border
call PrintString(1,69,"RET=Select")

color Highcolor,Background,Border
x=int((80-len(ArcName$))/2)
call PrintString(2,x,ArcName$)

color Forground,Background,Border
call PrintString(2,69,"END=Run")
RETURN

'===================================================================
ClearPage:

for i=3 to 25
    call PrintString(i,1,string$(79,32))
next i
RETURN

'===================================================================
DisplayPage:

gosub ClearPage

a$ = "PAGE=" + right$(str$(page+1),len(str$(page+1))-1)
call PrintString(2,1,a$)
call PrintString(25,1,"F1=Help")
call PrintString(25,69,"F10=Update")

y=4
for i=1 to 20
    j = (page*80)+i
    call PrintString(y,3,CourseNames$(j))
    call PrintString(y,23,CourseNames$(j+20))
    call PrintString(y,43,CourseNames$(j+40))
    call PrintString(y,63,CourseNames$(j+60))
    y = y + 1
next i

Reverse=true
gosub ReverseCourse
RETURN

'==================================================================
CopyRightScreen:

data "                M18 (c) 1988 by William T. Faulkner"
data "                        All Rights Reserved"
data " "
data " "
data "    Faulkner Consulting, 4552 N. 76th Street, Milwaukee, WI  53218"
data " "
data "Faulkner Consulting grants the users of M18 the freedom to distribute"
data "M18 and its documentation files without charge so long as no payment"
data "is received by the user for M18.  If you wish to charge a fee for M18"
data "please contact Faulkner Consulting for written approval."

restore CopyRightScreen                          'Print Copyright Info
for y=7 to 16
    read a$
    call PrintString(y,6,a$)
next y
RETURN

'==================================================================
DisplayHelp:

data "M18.EXE [CourseName]   -  The Golf Course Archive Name is optional."
data " "
data "M18 is a shell program for Mean-18 by Accolade(tm).  Mean-18 is by far"
data "the best user supported golf game for the IBM.  However Mean-18 has 3"
data "major problems, two of which are addressed by this program.  One;"
data "Mean-18 only looks for the first 3 golf courses on the current logged"
data "drive.  Two; Mean-18 is copy protected and can not be run from a hard"
data "disk drive.  Three; Mean-18 golf courses (100's available) require over"
data "110K of valuable disk space each.  M18 addresses problems #1 and #3."
data "To solve the copy protection problem #2, I recommend a program called"
data "MEAN.ARC which is available on EXEC-PC (414) 964-5160 or check your"
data "favorite BBS for a simular breaker program."
data " "
data "M18 works best on a hard disk system but a two floppy system can be"
data "used.  M18 can handle up to 400 golf courses.  By using archive files"
data "you can expect a 2/3 space savings on your hard disk.  The following 5"
data "files are needed by M18 in order for it to work correctly."
data " "
data "M18.EXE       - The shell program."
data "M18.DIR       - Created by M18 & PKARC."
data "COURSES.ARC   - The golf courses archive file."
data "PKARC.COM     - The Archiver program from:     PKWARE Inc. 7032 Ardara Ave."
data "PKXARC.COM    - The Unarchiver program.        Glendale, Wisconsin  53209"

cls                                              'Setup Help Screen
call PrintString(1,1,"ESC=Exit Help")

color Background,Forground,Border
call PrintString(1,26," GOLF COURSE SELECTOR "+version$)

color Forground,Background,Border
call PrintString(2,1,string$(79,"-"))

color Highcolor,Background,Border
restore DisplayHelp                              'Print Help Info
for y=3 to 25
    read a$
    call PrintString(y,5,a$)
next y
color Forground,Background,Border

a$=""
while a$<>EscKey$                                'loop until ESC pressed
    while not instat : wend                      'wait for key
    a$ = inkey$                                  'get key
    if a$<>EscKey$ then Beep                     'wrong key pressed
wend
RETURN

'==================================================================
GetChar:

color Highcolor,Background,Border
call PrintString(25,30," WAITING FOR A KEY ")
color Forground,Background,Border

found = false
while not found
    while not instat : wend                      'wait for key
    choice$ = inkey$                             'get key
    select case choice$                          'check if valid
        case UpArrow$: found = true
        case DnArrow$: found = true
        case LftArrow$: found = true
        case RgtArrow$: found = true
        case EscKey$: found = true
        case RetKey$: found = true
        case EndKey$: found = true
        case PgUp$: found = true
        case PgDn$: found = true
        case F1$: found = true
        case F10$: found = true
        case else : beep
    end select                                   'Invalid Key Pressed
wend

call PrintString(25,30,"                   ")
RETURN

'================================================================
UpdateArchive:

gosub SetupScreen                                'Setup Display

color Highcolor,Background,Border                'Highlight Messages

if DoAllFiles then
    call PrintString(25,30," SAVING COURSE(S) ")
    shell "PKARC u "+ArcName$+" *.M18 >"+TempName$
end if

call PrintString(25,30," SAVING SAVED GAME(S) ")
shell "PKARC u "+ArcName$+" *.SAV >"+TempName$

call PrintString(25,30," SAVING HALL OF FAME(S) ")
shell "PKARC u "+ArcName$+" *.HOF >"+TempName$

kill TempName$                                   'Clean up Disk
kill "*.M18"
kill "*.HOF"
kill "*.SAV"

color Forground,Background,Border
call PrintString(25,30,"                        ")
RETURN

'=================================================================
GetCourseNames:

color Highcolor,Background,Border
call PrintString(25,26," BUILDING COURSES DIRECTORY ")

for i=1 to max                                       'clear array
    CourseNames$(i)=""
next i

ShellCommand$ = "PKARC v "+ArcName$+" >"+DirName$
shell ShellCommand$                                  'get directory

count = 0
open DirName$ for INPUT as #1                        'put courses in array
    while not(EOF(1))
        line input #1, a$
        a = instr(a$,".M18")
        if (a>1) and (a<10) then
            count = count + 1
            CourseNames$(count) = left$(a$,a-1) + " (C"
        end if
    wend
close #1

open DirName$ for INPUT as #1                        'check for Hall of Fames
    while not(EOF(1))
        line input #1, a$
        a = instr(a$,".HOF")
        if (a>1) and (a<10) then
            a$ = left$(a$,a-1) + " (C"
            for i = 1 to count
                if a$ = CourseNames$(i) then_
                    CourseNames$(i) = CourseNames$(i) + "H"
            next i
        end if
    wend
close #1

open DirName$ for INPUT as #1                        'check for Saved Games
    while not(EOF(1))
        line input #1, a$
        a = instr(a$,".SAV")
        if (a>1) and (a<10) then
            a1$ = left$(a$,a-1) + " (C"
            a$ = left$(a$,a-1) + " (CH"
            for i = 1 to count
                if a1$=CourseNames$(i) or a$=CourseNames$(i) then_
                    CourseNames$(i) = CourseNames$(i) + "S"
            next i
        end if
    wend
close #1

call PrintString(25,26,"  SORTING COURSES DIRECTORY ")

swapped=true                                         'sort course names
while swapped
    swapped = false
    for i = 1 to count-1
        if CourseNames$(i) > CourseNames$(i+1) then
            swap CourseNames$(i),CourseNames$(i+1)
            swapped = true
        end if
    next i
wend

posit = 0
for i = 1 to Max                                     'add line numbers
    posit = posit + 1
    if posit > 80 then posit = 1

    a$ = right$(str$(posit),len(str$(posit))-1)
    if len(a$) = 1 then a$="0"+a$+"- " else a$=a$+"- "

    if CourseNames$(i) = "" then
        CourseNames$(i) = a$ + CourseNames$(i)
    else
        CourseNames$(i) = a$ + CourseNames$(i) + ")"
    end if
next i

color Forground,Background,Border
call PrintString(25,26,"                            ")
NumCourses = Count                                   'Number of courses
RETURN

'=================================================================
BlinkPosition:

select case position                                 'determine x,y locations
  case < 21
    x = 3
    y = position + 3
  case < 41
    x = 23
    y = position - 17
  case < 61
    x = 43
    y = position - 37
  case else
    x = 63
    y = position - 57
end select

a$ = right$(str$(position),len(str$(position))-1)
if len(a$) = 1 then a$="0"+a$+"-" else a$=a$+"-"

if Blink then color Blnkcolor,Background,Border
call PrintString(y,x,a$)
color Forground,Background,Border
RETURN

'================================================================
ReverseCourse:

select case LastSelected                    'determine x,y locations
  case < 21
    x = 3
    y = LastSelected + 3
  case < 41
    x = 23
    y = LastSelected - 17
  case < 61
    x = 43
    y = LastSelected - 37
  case else
    x = 63
    y = LastSelected - 57
end select

if Reverse then color Background,Forground,Border
call PrintString(y,x,CourseNames$((page*80)+LastSelected))
color Forground,Background,Border
RETURN

'================================================================
SUB PrintString(linepos%,column%,prompt$)

locate linepos%,column%                          'Print prompt$ @ x,y
print prompt$;
END SUB

'================================================================
IgnoreErrors:

RESUME NEXT                                      'Ignore all errors
STOP

'==================================================================
ExitRoutine:

cls                                              'clear screen
locate ,,1                                       'Turn Cursor Back ON
END                                              'END PROGRAM

